/*
COPYRIGHT (C) 2016  Roberto Bucher (roberto.bucher@supsi.ch)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#include <pyblock.h>

#define BYTE unsigned char
#define WORD unsigned short
#define DWORD unsigned int

#define TIMEOUT -1

#include <canopen.h>
#include <stdlib.h>
#include <unistd.h>

static BYTE NMT_pre[2]  = {0x80, 0x00};
static BYTE NMT_act[2]  = {0x01, 0x00};

#define DATA_SIZE 8

static BYTE data[DATA_SIZE][8]= {
  {0x2b, 0x40, 0x60, 0x00, 0x06, 0x00, 0x00, 0x00},
  {0x2b, 0x40, 0x60, 0x00, 0x0F, 0x00, 0x00, 0x00},
  {0x2b, 0x32, 0x23, 0x01, 0x00, 0x00, 0x00, 0x00},
  {0x2b, 0x32, 0x23, 0x02, 0x00, 0x00, 0x00, 0x00},
  {0x2f, 0x60, 0x60, 0x00, 0x06, 0x00, 0x00, 0x00},
  {0x2f, 0x98, 0x60, 0x00, 0x23, 0x00, 0x00, 0x00},
  {0x2b, 0x40, 0x60, 0x00, 0x0f, 0x00, 0x00, 0x00},
  {0x2b, 0x40, 0x60, 0x00, 0x1f, 0x00, 0x00, 0x00},
};

/* ------------------------------------------------
0+1 Initialize
2+3 Pos Controller Values
4+5 Homing - set pos to 0
6-7 Set Control method
-------------------------------------------------*/

static BYTE pos_mode[8]    = {0x2f, 0x60, 0x60, 0x00, 0x01, 0x00, 0x00, 0x00};
static BYTE write_value[8] = {0x23, 0x7a, 0x60, 0x00, 0x00, 0x00, 0x00, 0x00};
static BYTE write_init[8]  = {0x2b, 0x40, 0x60, 0x00, 0x0f, 0x00, 0x00, 0x00};
static BYTE write_exec[8]  = {0x2b, 0x40, 0x60, 0x00, 0x3f, 0x00, 0x00, 0x00};
static BYTE disable[8]     = {0x2b, 0x40, 0x60, 0x00, 0x06, 0x00, 0x00, 0x00};

/*-------------------------------------------------
block->intPar[0]: ID
block->intPar[1]: If 1 -> homing set 0
block->realPar[0]: res
block->realPar[1]: kp
block->realPar[2]: kd
-------------------------------------------------*/

static void init(python_block *block)
{
  int i;
  BYTE DATA[8];
  unsigned char cval;
  int *U_can;
  
  if(canOpenTH()) exit(1);

  NMT_pre[1] = (BYTE) block->intPar[0];
  NMT_act[1] = (BYTE) block->intPar[0];

  /* sendMsg(0x000,NMT_pre,2); */
  sendMsg(0x000,NMT_act,2);   /* Operational status */

  for(i=0;i<2;i++){
    sendMsg(0x600+block->intPar[0],data[i],8);  /* configure device */
    usleep(50000);
  }

  if (block->realPar[0] > 0.0){
      cval = (unsigned char) ((int) block->realPar[0] & 0x0000FFFF);
      data[2][4] = cval;
      cval = (unsigned char) ((int) block->realPar[1] & 0x0000FFFF);
      data[3][4] = cval;
      for(i=2;i<4;i++){
	sendMsg(0x600+block->intPar[0],data[i],8);  /* Set this position to 0 */
	usleep(50000);
      }
  }

  if(block->intPar[1]){
    for(i=4;i<6;i++){
      sendMsg(0x600+block->intPar[0],data[i],8);  /* Set this position to 0 */
      usleep(50000);
    }
  }

  for(i=6;i<DATA_SIZE;i++){
    sendMsg(0x600+block->intPar[0],data[i],8);
    usleep(50000);
  }

  sendMsg(0x600+block->intPar[0],pos_mode,8); 
  usleep(100000);
}

static void inout(python_block *block)
{
  int *U_can;

  BYTE DATA[8];
  double *u = block->u[0];
  unsigned short *index;
  
  U_can = (int *) &write_value[4];
  *U_can = (int) (u[0]*block->realPar[0]);

  sendMsg(0x600+block->intPar[0],write_value,8);
  sendMsg(0x600+block->intPar[0],write_init,8);
  sendMsg(0x600+block->intPar[0],write_exec,8);
}

static void end(python_block *block)
{
  canClose();
}

void FH_can_motX(int Flag, python_block *block)
{
  if (Flag==OUT){          /* set output */
    inout(block);
  }
  else if (Flag==END){     /* termination */ 
    end(block);
  }
  else if (Flag ==INIT){    /* initialisation */
    init(block);
  }
}


